import {
  select,
  csv,
  scaleBand,
  max,
  scaleLinear,
  scaleOrdinal,
  extent,
  axisBottom,
  axisLeft,
  //schemeCategory10,
  format,
} from 'd3';

export const horizontalChart = (selection, props) => {
  const {
    margin,
    width,
    height,
    total,
    data,
    attributes,
    xAxisLabel,
    yAxisLabel,
    onTypeSelect,
  } = props;

  //console.log(data);

  const innerWidth =
    width - margin.right - margin.left - 50;
  const innerHeight = height - margin.top - margin.bottom;

  const g = selection.selectAll('.container').data([null]);
  const gEnter = g
    .enter()
    .append('g')
    .attr('class', 'container');
  gEnter
    .merge(g)
    .attr(
      'transform',
      `translate(${margin.left},${margin.top})`,
    );

  const allNames = Object.keys(attributes);
  const allColors = Object.values(attributes);

  const xScale = scaleLinear()
    .domain([0, max(Object.values(data))])
    .range([0, innerWidth]);

  const yScale = scaleBand()
    .domain(allNames)
    .range([0, innerHeight])
    .padding(0.1);

  const xAxisG = g.select('.x-axis');
  const xAxisGEnter = gEnter
    .append('g')
    .attr('class', 'x-axis');
  xAxisG
    .merge(xAxisGEnter)
    .attr('transform', `translate(0,${innerHeight})`)
    .call(
      axisBottom(xScale)
        .tickPadding(10)
        .tickSize(-innerHeight)
        .ticks(6),
    );
  const xAxisLabelText = xAxisGEnter
    .append('text')
    .attr('class', 'axis-label')
    .attr('y', 50)
    .attr('fill', 'black')
    .merge(xAxisG.select('.axis-label'))
    .attr('x', innerWidth / 2)
    .text(xAxisLabel);

  const yAxisG = g.select('.y-axis');
  const yAxisGEnter = gEnter
    .append('g')
    .attr('class', 'y-axis');
  yAxisG
    .merge(yAxisGEnter)
    .call(
      axisLeft(yScale)
        .tickPadding(10)
        .tickSize(-innerWidth),
    )
    .selectAll('.tick line') // Select tick lines
    .remove();
  const yAxisLabelText = yAxisGEnter
    .append('text')
    .attr('class', 'axis-label')
    .attr('y', -50)
    .attr('transform', 'rotate(-90)')
    .attr('text-anchor', 'middle')
    .attr('fill', 'black')
    .merge(yAxisG.select('.axis-label'))
    .attr('x', -innerHeight / 2)
    .text(yAxisLabel);

  const rects = g
    .merge(gEnter)
    .selectAll('rect')
    .data(allNames);
  rects
    .enter()
    .append('rect')
    .attr('x', xScale(0))
    .attr('y', function (d) {
      return yScale(d);
    })
    .attr('width', function (d) {
      return xScale(data[d]);
    })
    .attr('height', yScale.bandwidth())
    .attr('fill', function (d) {
      return attributes[d];
    })
    .attr('class', function (d) {
      return `bar-${d}`;
    })
    .on('mouseover', handleMouseOver)
    .on('mouseout', handleMouseOut)
    .on('click', handleMouseClick);

  let selectedTypes = [];

  function handleMouseOver(d) {
    // Darken the current type
    select(this)
      .attr('fill', darkenColor(attributes[d], 0.8))
      .attr('opacity', 1);

    //console.log(total);

    select(this)
      .append('title')
      .text(
        `Type: ${d}\nCount: ${data[d]} (${format('.0%')(
          data[d] / total,
        )})`,
      );

    onTypeSelect(d);
    //console.log(selectedTypes);
  }

  function handleMouseClick(d) {
    if (!selectedTypes.includes(d)) {
      selectedTypes.push(d);
    }
    if (selectedTypes.length > 2) {
      selectedTypes.shift();
    }
    allNames.forEach((type) => {
      if (!selectedTypes.includes(type)) {
        select(`.bar-${type}`)
          .attr('fill', attributes[type])
          .attr(
            'opacity',
            selectedTypes.length >= 1 ? 0.3 : 1,
          );
      } else {
        select(`.bar-${type}`).attr('opacity', 1);
      }
    });
    onTypeSelect(selectedTypes);
  }

  function darkenColor(color, factor) {
    return d3.color(color).darker(factor).toString();
  }

  function handleMouseOut(d) {
    // Reset opacity on mouseout
    allNames.forEach((type) => {
      if (!selectedTypes.includes(type)) {
        select(`.bar-${type}`)
          .attr('fill', attributes[type])
          .attr(
            'opacity',
            selectedTypes.length >= 1 ? 0.3 : 1,
          );
      } else {
        select(`.bar-${type}`).attr('opacity', 1);
      }
    });
    onTypeSelect(selectedTypes);
    //select(this).attr('fill', attributes[d]);
  }
};
