import {
  select,
  csv,
  scaleLinear,
  scaleOrdinal,
  extent,
  axisBottom,
  axisLeft,
  format,
  //forceSimulation,
  //forceCollide,
  //forceX,
  //forceY,
  //schemeCategory10,
} from 'd3';
import { colorLegend } from './colorLegend';

export const scatterPlot = (selection, props) => {
  const {
    margin,
    width,
    height,
    data,
    xValue,
    xAxisLabel,
    yValue,
    yAxisLabel,
    colorValue,
    colorLegendLabel,
    colorRange,
    addition,
    number,
    fastOrCharge,
  } = props;

  const innerWidth =
    width - margin.right - margin.left - 50;
  const innerHeight = height - margin.top - margin.bottom;

  const g = selection.selectAll('.container').data([null]);
  const gEnter = g
    .enter()
    .append('g')
    .attr('class', 'container');
  gEnter
    .merge(g)
    .attr(
      'transform',
      `translate(${margin.left},${margin.top})`,
    );

  const xScale = scaleLinear()
    .domain(extent(data, xValue))
    .range([0, innerWidth])
    .nice();

  const yScale = scaleLinear()
    .domain(extent(data, yValue))
    .range([innerHeight, 0])
    .nice();
  /*
  const colorScale = scaleOrdinal()
    .domain(data.map(colorValue))
    .range(colorRange);
*/
  const allTypes = Object.keys(colorRange);
  let selectedTypes = [];

  const colorScale = scaleOrdinal()
    .domain(allTypes) // Use the types as the domain
    .range(Object.values(colorRange)); // Use the corresponding colors as the range

  const xAxisG = g.select('.x-axis');
  const xAxisGEnter = gEnter
    .append('g')
    .attr('class', 'x-axis');
  xAxisG
    .merge(xAxisGEnter)
    .attr('transform', `translate(0,${innerHeight})`)
    .call(
      axisBottom(xScale)
        .tickPadding(10)
        .tickSize(-innerHeight),
    );
  const xAxisLabelText = xAxisGEnter
    .append('text')
    .attr('class', 'axis-label')
    .attr('y', 60)
    .attr('fill', 'black')
    .merge(xAxisG.select('.axis-label'))
    .attr('x', innerWidth / 2)
    .text(xAxisLabel);

  const yAxisG = g.select('.y-axis');
  const yAxisGEnter = gEnter
    .append('g')
    .attr('class', 'y-axis');
  yAxisG
    .merge(yAxisGEnter)
    .call(
      axisLeft(yScale)
        .tickPadding(10)
        .tickSize(-innerWidth)
        .ticks(5),
    );
  const yAxisLabelText = yAxisGEnter
    .append('text')
    .attr('class', 'axis-label')
    .attr('y', -50)
    .attr('transform', 'rotate(-90)')
    .attr('text-anchor', 'middle')
    .attr('fill', 'black')
    .merge(yAxisG.select('.axis-label'))
    .attr('x', -innerHeight / 2)
    .text(yAxisLabel);

  selection.call(colorLegend, {
    colorScale,
    colorLegendLabel,
  });

  // Function to add jitter to a value within chart boundaries
  function addJitter(value, jitterAmount, chartSize) {
    const jitter = (Math.random() - 0.5) * jitterAmount;
    const jitteredValue = value + jitter;
    return Math.max(Math.min(jitteredValue, chartSize), 0);
  }

  // Jitter amount (adjust as needed)
  const jitterAmount = 20;

  console.log(addition);

  const moveCounts = Array.from(addition.entries()).map(
    ([move, pokemonList]) => {
      return {
        move: move,
        count: pokemonList.length,
      };
    },
  );
  console.log(data);

  const circles = g
    .merge(gEnter)
    .selectAll('circle')
    .data(data);
  circles
    .enter()
    .append('circle')
    .attr('class', (d) => d.Types)
    //.attr('cx', innerWidth / 2)
    //.attr('cy', innerHeight / 2)
    //.attr('r', 0)
    //.transition()
    //.duration(2000)
    //.delay((d, i) => i * 10)
    .attr('cy', (d) =>
      addJitter(
        yScale(yValue(d)),
        jitterAmount,
        innerHeight,
      ),
    )
    .attr('cx', (d) =>
      addJitter(
        xScale(xValue(d)),
        jitterAmount,
        innerWidth,
      ),
    )
    .attr('r', (d) => {
      // Find the corresponding count for the move
      const moveCount = moveCounts.find(
        (entry) => entry.move === d.Moves,
      );
      return moveCount
        ? number > 150
          ? moveCount.count * 1
          : moveCount.count * 5
        : 0; // Set radius to count * 1.5 or 0 if not found
    })
    .attr('fill', (d) => colorScale(colorValue(d)))
    .attr('opacity', 0.6)
    .on('mouseover', handleMouseOver)
    .on('mouseout', handleMouseOut);

  function handleMouseOver(d, event) {
    // Darken the current type
    //console.log(d);
    select(this)
      .attr('fill', darkenColor(colorScale(d.Types), 0.8))
      .append('title')
      .text((d) => {
        if (fastOrCharge == 'fast') {
          return `${d.Moves}\nDPT: ${format('.1f')(
            d.DPT,
          )}\nEPT: ${format('.1f')(d.EPT)}\nTurn:${d.Turn}\nPokémon: ${getPokemonList(
            d.Moves,
          )}`;
        } else {
          return `${d.Moves}\nBase Power: ${
            d['Base Power']
          }\nEnergy Cost: ${
            d.Energy
          }\nPokémon: ${getPokemonList(d.Moves)}`;
        }
      });
  }

  function darkenColor(color, factor) {
    return d3.color(color).darker(factor).toString();
  }

  function getPokemonList(move) {
    const pokemonList = addition.get(move);
    return pokemonList
      ? pokemonList
          .map(
            (pokemon) =>
              `${pokemon.name} (${pokemon.rank})`,
          )
          .join('\n')
      : 'No Pokémon found';
  }

  function handleMouseOut(d) {
    select(this).attr('fill', colorScale(d.Types));
  }

  allTypes.forEach((type) => {
    //console.log(type);
    selection
      //.selectAll('.tick')
      .selectAll(`.${type}-legend`)
      .on('mouseover', MouseOverType)
      .on('click', ClickType)
      .on('mouseout', MouseOutType);
  });

  function MouseOverType(d) {
    select(this)
      .selectAll('circle')
      .attr('fill', darkenColor(colorScale(d), 1.0));
    selection.selectAll(`.${d}`).attr('opacity', 0.8);
    allTypes.forEach((type) => {
      if (type != d && !selectedTypes.includes(type)) {
        selection
          .selectAll(`.${type}`)
          .attr('opacity', 0.1);
      }
    });
  }

  function ClickType(d) {
    if (!selectedTypes.includes(d)) {
      selectedTypes.push(d);
    }
    allTypes.forEach((type) => {
      /*
      if (!selectedTypes.includes(type)) {
        selection
          .selectAll(`.${type}`)
          .attr('opacity', 0.05);
      } else {
        //console.log(type);
        selection
          .selectAll(`.${type}`)
          .attr('opacity', 0.6);
      }*/
      if (!selectedTypes.includes(type)) {
        selection
          .selectAll(`.${type}-legend`)
          .attr('opacity', 0.2);
      } else {
        selection
          .selectAll(`.${type}-legend`)
          .attr('opacity', 1);
      }
      const selected = selectedTypes.includes(type);
      selection
        .selectAll(`.${type}`)
        .style('display', selected ? 'block' : 'none');
    });
  }

  function MouseOutType(d) {
    console.log(selectedTypes);
    if (!selectedTypes.includes(d)) {
      select(this)
        .selectAll('circle')
        .attr('fill', colorScale(d));
      selection.selectAll(`.${d}`).attr('opacity', 0.05);
    }
    if (selectedTypes.length == 0) {
      allTypes.forEach((type) => {
        if (type != d) {
          selection
            .selectAll(`.${type}`)
            .attr('opacity', 0.6);
        }
      });
    }
  }
